--- u-boot/drivers/serial.c	Fri Aug 29 15:00:47 2003
+++ bios/drivers//serial.c	Thu Oct 21 09:02:14 2004
@@ -30,28 +30,61 @@
 #include <ns87308.h>
 #endif
 
-#if CONFIG_CONS_INDEX == 1
-static NS16550_t console = (NS16550_t) CFG_NS16550_COM1;
-#elif CONFIG_CONS_INDEX == 2
-static NS16550_t console = (NS16550_t) CFG_NS16550_COM2;
-#elif CONFIG_CONS_INDEX == 3
-static NS16550_t console = (NS16550_t) CFG_NS16550_COM3;
-#elif CONFIG_CONS_INDEX == 4
-static NS16550_t console = (NS16550_t) CFG_NS16550_COM4;
+#if !defined(CONFIG_CONS_INDEX)
+#error	"No console index specified."
+#elif (CONFIG_CONS_INDEX < 1) || (CONFIG_CONS_INDEX > 4)
+#error	"Invalid console index value."
+#endif
+
+#if CONFIG_CONS_INDEX == 1 && !defined(CFG_NS16550_COM1)
+#error	"Console port 1 defined but not configured."
+#elif CONFIG_CONS_INDEX == 2 && !defined(CFG_NS16550_COM2)
+#error	"Console port 2 defined but not configured."
+#elif CONFIG_CONS_INDEX == 3 && !defined(CFG_NS16550_COM3)
+#error	"Console port 3 defined but not configured."
+#elif CONFIG_CONS_INDEX == 4 && !defined(CFG_NS16550_COM4)
+#error	"Console port 4 defined but not configured."
+#endif
+
+/* Note: The port number specified in the functions is 1 based.
+ *	 the array is 0 based.
+ */
+static NS16550_t serial_ports[4] = {
+#ifdef CFG_NS16550_COM1
+	(NS16550_t)CFG_NS16550_COM1,
 #else
-#error no valid console defined
+	NULL,
 #endif
+#ifdef CFG_NS16550_COM2
+	(NS16550_t)CFG_NS16550_COM2,
+#else
+	NULL,
+#endif
+#ifdef CFG_NS16550_COM3
+	(NS16550_t)CFG_NS16550_COM3,
+#else
+	NULL,
+#endif
+#ifdef CFG_NS16550_COM4
+	(NS16550_t)CFG_NS16550_COM4
+#else
+	NULL
+#endif
+};
+
+#define PORT	serial_ports[port-1]
+#define CONSOLE	(serial_ports[CONFIG_CONS_INDEX-1])
 
-static int calc_divisor (void)
+static int calc_divisor (NS16550_t port)
 {
 	DECLARE_GLOBAL_DATA_PTR;
 #ifdef CONFIG_OMAP1510
 	/* If can't cleanly clock 115200 set div to 1 */
 	if ((CFG_NS16550_CLK == 12000000) && (gd->baudrate == 115200)) {
-		console->osc_12m_sel = OSC_12M_SEL;	/* enable 6.5 * divisor */
+		port->osc_12m_sel = OSC_12M_SEL;	/* enable 6.5 * divisor */
 		return (1);				/* return 1 for base divisor */
 	}
-	console->osc_12m_sel = 0;			/* clear if previsouly set */
+	port->osc_12m_sel = 0;			/* clear if previsouly set */
 #endif
 #ifdef CONFIG_OMAP1610
 	/* If can't cleanly clock 115200 set div to 1 */
@@ -64,54 +97,111 @@
 
 int serial_init (void)
 {
-	int clock_divisor = calc_divisor();
+	int clock_divisor;
 
 #ifdef CFG_NS87308
 	initialise_ns87308();
 #endif
 
-	NS16550_init(console, clock_divisor);
+#ifdef CFG_NS16550_COM1
+	clock_divisor = calc_divisor(serial_ports[0]);
+	NS16550_init(serial_ports[0], clock_divisor);
+#endif
+#ifdef CFG_NS16550_COM2
+	clock_divisor = calc_divisor(serial_ports[1]);
+	NS16550_init(serial_ports[1], clock_divisor);
+#endif
+#ifdef CFG_NS16550_COM3
+	clock_divisor = calc_divisor(serial_ports[2]);
+	NS16550_init(serial_ports[2], clock_divisor);
+#endif
+#ifdef CFG_NS16550_COM4
+	clock_divisor = calc_divisor(serial_ports[3]);
+	NS16550_init(serial_ports[3], clock_divisor);
+#endif
 
 	return (0);
 }
 
 void
-serial_putc(const char c)
+_serial_putc(const char c,const int port)
 {
 	if (c == '\n')
-		NS16550_putc(console, '\r');
+		NS16550_putc(PORT, '\r');
 
-	NS16550_putc(console, c);
+	NS16550_putc(PORT, c);
 }
 
 void
-serial_puts (const char *s)
+_serial_putc_raw(const char c,const int port)
+{
+	NS16550_putc(PORT, c);
+}
+
+void
+_serial_puts (const char *s,const int port)
 {
 	while (*s) {
-		serial_putc (*s++);
+		_serial_putc (*s++,port);
 	}
 }
 
 
 int
-serial_getc(void)
+_serial_getc(const int port)
 {
-	return NS16550_getc(console);
+	return NS16550_getc(PORT);
 }
 
 int
-serial_tstc(void)
+_serial_tstc(const int port)
 {
-	return NS16550_tstc(console);
+	return NS16550_tstc(PORT);
 }
 
 void
-serial_setbrg (void)
+_serial_setbrg (const int port)
 {
 	int clock_divisor;
 
-    clock_divisor = calc_divisor();
-	NS16550_reinit(console, clock_divisor);
+	clock_divisor = calc_divisor(PORT);
+	NS16550_reinit(PORT, clock_divisor);
+}
+
+void 
+serial_putc(const char c)
+{
+	_serial_putc(c,CONFIG_CONS_INDEX);
+}
+
+void 
+serial_putc_raw(const char c)
+{
+	_serial_putc_raw(c,CONFIG_CONS_INDEX);
+}
+
+void 
+serial_puts(const char *s)
+{
+	_serial_puts(s,CONFIG_CONS_INDEX);
+}
+
+int 
+serial_getc(void)
+{
+	return _serial_getc(CONFIG_CONS_INDEX);
+}
+
+int 
+serial_tstc(void)
+{
+	return _serial_tstc(CONFIG_CONS_INDEX);
+}
+
+void 
+serial_setbrg(void)
+{
+	_serial_setbrg(CONFIG_CONS_INDEX);
 }
 
 #endif
